package F5::ASMConfig::Handler::ExportSuggestionsTask;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::TaskBase/;
use F5::ASMConfig::Entity::ExportSuggestionsTask;
use F5::PopulateTemplate;
use JSON;
use F5::Utils::Rest;
use File::Slurp;

our @handled_functions = (
                            {
                             function_name => 'add_export_suggestions_task',
                             criteria_index => 3,
                             store_index    => 4,
                            },
                            {
                              function_name => 'update_export_suggestions_task',
                              criteria_index => 3,
                              store_index => 4
                            },
                            {
                             function_name => 'execute_export_suggestions_task',
                            },
                            {
                             function_name => 'delete_export_suggestions_task',
                            },
                            {
                             function_name => 'delete_many_export_suggestions_tasks',
                            },
                         );

our $entity_class = 'F5::ASMConfig::Entity::ExportSuggestionsTask';

my $BASE_URI = '/mgmt/tm/asm/policies';
my $SUGGESTIONS_URI = 'suggestions';
my $STATUS_FILTER = "status ne 'ignored'";

#----------------------------------------------------------------------

sub add_export_suggestions_task {
  my($self, @rpc_params) = @_;
  return F5::ASMConfig::Handler::TaskBase::add_task(self => $self,
                                                    rpc_params => \@rpc_params,
                                                    entity_class => $entity_class,
                                                    task_type => 'export_suggestions',);
}

#----------------------------------------------------------------------

sub update_export_suggestions_task {
    my ($self, @rpc_params) = @_;
    return F5::ASMConfig::Handler::TaskBase::update_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                         task_type => 'export_suggestions',);
}

#----------------------------------------------------------------------

sub export_suggestions {
    my (%params) = @_;
    my $self       = $params{self};
    my $task       = $params{task};
    my $logger     = $self->{logger};

    my $filename;
    if ($task->{filename}) {
        $filename = F5::Utils::Rest::get_full_file_path(username => $task->{username}, filename => ($task->{filename} || $task->{rest_uuid}) );
    }

    my $policy = $task->{policy};
    my $rest_path = "$BASE_URI/$policy->{rest_uuid}/$SUGGESTIONS_URI";

    my @select_fields = ( qw/entityReference entityChanges entityKind action description/ );

    my $filter = $STATUS_FILTER;
    if ($task->{filter}) {
        $filter = "$filter and $task->{filter}"
    }

    my $rest_request = F5::ASMConfig::RestRequest->new(rest_path => $rest_path,
                                                       rest_method => 'GET',
                                                       rest_parameters => { '$select' => join(',', @select_fields),
                                                                            '$filter' => $filter,
                                                                          }
                                                      );

    my $call_flags = F5::ASMConfig::CallFlags->new();

    my $rv;
    eval {
        $rv = $self->rest_call($self->{session}, $call_flags, $rest_request);
        $rv = decode_json($rv);

        for my $item (@{$rv->{items}}) {
            delete $item->{selfLink};
            delete $item->{kind};

            my $entity_type = F5::YAML::Schema::Utils::get_entity_type(entity_kind => $item->{entityKind});

            $item->{entityType} = $entity_type->element_rest_name;
            delete $item->{entityKind};

            my $entity_changes = $item->{entityChanges};
            if (defined $entity_changes) {
                delete $entity_changes->{lastUpdateMicros};

                if (!keys %{$entity_changes}) {
                    delete $item->{entityChanges};
                }
            }

            F5::YAML::Schema::Utils::item_references_fixup(item => $item->{entityReference},
                                                           entity_type => $entity_type,
                                                           direction => 'to_declarative',
                                                          );
            $item->{entity} = $item->{entityReference};
            delete $item->{entityReference};

            next if !$item->{entityChanges};

            F5::YAML::Schema::Utils::item_references_fixup(item => $item->{entityChanges},
                                                           entity_type => $entity_type,
                                                           direction => 'to_declarative',
                                                          );

        }

        my $suggestions = encode_json($rv->{items});

        if ($task->{result_inline}) {
            $task->{exported_suggestions} = $suggestions;
        }
        else {
            $task->{file_size} = length($suggestions);
            File::Slurp::write_file($filename, $suggestions);
        }
    };
    if ($@) {
        $task->{message} = ref $@ ? $@->{error_message} : $@;
        $task->{status} = F5::ASMConfig::TaskStatus::TASK_STATUS_FAILURE();
    }
    else {
        $task->{status} = F5::ASMConfig::TaskStatus::TASK_STATUS_COMPLETED();
    }
}

#----------------------------------------------------------------------

sub execute_export_suggestions_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::execute_task(self => $self,
                                                          rpc_params => \@rpc_params,
                                                          entity_class => $entity_class,
                                                          task_type => 'export_suggestions',
                                                          execute_func => sub { export_suggestions(@_); },
                                                          );
}

#----------------------------------------------------------------------

sub delete_export_suggestions_task {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_task(self => $self,
                                                         rpc_params => \@rpc_params,
                                                         entity_class => $entity_class,
                                                        );
}

#----------------------------------------------------------------------

sub delete_many_export_suggestions_tasks {
    my ($self, @rpc_params) = @_;

    return F5::ASMConfig::Handler::TaskBase::delete_many_tasks(self => $self,
                                                               rpc_params => \@rpc_params,
                                                               entity_class => $entity_class,
                                                              );
}

#----------------------------------------------------------------------

1;
