package F5::YAML::Schema::Utils;

use strict;
use warnings;

use File::Slurp;
use YAML::XS;
use JSON;
use F5::Deploy;
use F5::ASMConfig::Entity::Base;

my $entity_references = {};
my $entity_kind_map = {};
my $reference_name_map = {};
my $YAML_TO_JSON_APP = F5::Deploy::map_path('/usr/share/ts/bin/yaml2json');

#----------------------------------------------------------------------

sub parse_json {
    my(%params) = @_;

    my $rv;

    return if !($params{content} || $params{filename});

    my $content = $params{content} || read_file($params{filename});
    eval {
        $rv->{data} = decode_json($content);
        $rv->{completed_successfully} = 1;
    };
    if (my $err = $@) {
        $err =~ s/at \/.*//;
        $rv->{completed_successfully} = 0;
        $rv->{error_message} = $err;
    }

    return $rv;
}

#----------------------------------------------------------------------

sub parse_yaml {
    my(%params) = @_;

    my $rv;

    return if !($params{content} || $params{filename});

    my $content = $params{content} || read_file($params{filename});
    eval {
        $rv->{data} = Load($content);
        $rv->{completed_successfully} = 1;
    };
    if (my $err = $@) {
        $rv->{completed_successfully} = 0;
        $rv->{error_message} = $err;
    }

    return $rv;
}

#----------------------------------------------------------------------

sub get_entity_structs {
    my(%params) = @_;

    my $entity_type = $params{entity_type};

    if (!$entity_references->{$entity_type}) {
        my $refs_map = $F5::ASMConfig::Handler::Rest::path_mapper->{references}->{policies}->{references};
        my %entity_to_ref_map = map { $refs_map->{"$_"}->{entity_type} => "$_"} grep { $refs_map->{$_}->{entity_type} } keys %$refs_map;

        my $entity_field_map = $entity_type->rest_field_map();
        my $fields = $entity_type->rest_fields();

        my @rest_external_structs = @{$entity_type->rest_external_struct_fields};
        map { $_ =~ s/__/\//g } @rest_external_structs;
        my $entity_external_structs = [ @rest_external_structs,
                                        keys %{$entity_type->all_external_structs} ];

        for my $external_struct (@$entity_external_structs) {
            my ($field, $sub_field) = split('/', $external_struct, 2);
            my $field_def = $sub_field ? $fields->{$field}->{fields}->{$sub_field} : $fields->{$field};

            next if $field_def->{rest_value_func} && $field_def->{rest_value_func} eq 'map_rest_external_field';

            my $external_struct_entity = $field_def->{rest_entity_type} || $field_def->{entity_type};
            my $unique_record_parts = $external_struct_entity->unique_record_parts;
            my $unique_record_parts_count = scalar @$unique_record_parts;
            my $unique_field;
            if ($unique_record_parts_count == 1) {
                $unique_field = $unique_record_parts->[0];
            }
            elsif (grep { $_ eq $external_struct_entity->id_field } @$unique_record_parts) {
                $unique_field = $external_struct_entity->id_field;
            }

            my $rest_unique_field_name = $unique_field ? $external_struct_entity->rest_field_map->{$unique_field} : undef;
            my $rest_sub_field_name = $sub_field ? $field_def->{rest_field_name} : undef;
            my $rest_sub_field_expand_name = $sub_field ? $field_def->{rest_expand_name} : undef;

            $entity_references->{$entity_type}->{$external_struct} = { field => $field,
                                                                       rest_field_name => $entity_field_map->{$field},
                                                                       sub_field => $sub_field,
                                                                       rest_sub_field_name => $rest_sub_field_name,
                                                                       rest_unique_field_name => $rest_unique_field_name,
                                                                       rest_group => $fields->{$field}->{rest_group},
                                                                       policy_reference => $entity_to_ref_map{$external_struct_entity},
                                                                       nested_struct => $fields->{$field}->{nested_struct},
                                                                       rest_expand_name => $fields->{$field}->{rest_expand_name},
                                                                       rest_sub_field_expand_name => $rest_sub_field_expand_name,
                                                                       skip_declarative_fixup => $field_def->{skip_declarative_fixup},
                                                                       declarative_fixup => $field_def->{declarative_fixup},
                                                                     };
        }
    }

    return $entity_references->{$entity_type};
}

#----------------------------------------------------------------------

sub get_entity_type {
    my(%params) = @_;

    my $kind = $params{entity_kind};

    if (!%$entity_kind_map) {
        my $refs_map = $F5::ASMConfig::Handler::Rest::path_mapper->{references}->{policies}->{references};
        for my $reference (values %$refs_map) {
            if ($reference->{entity_type}) {
                $entity_kind_map->{$reference->{kind_state}} = $reference->{entity_type};
                $entity_kind_map->{$reference->{element_rest_name}} = $reference->{entity_type};
            }
            elsif ($reference->{references}) {
                for my $sub_reference (values %{$reference->{references}}) {
                    $entity_kind_map->{$sub_reference->{kind_state}} = $sub_reference->{entity_type};
                    $entity_kind_map->{$sub_reference->{element_rest_name}} = $sub_reference->{entity_type};
                }
            }
        }
    }

    if ( $entity_kind_map->{$kind} ) {
        return $entity_kind_map->{$kind};
    }
    else{
        die "Entity Type/Kind '$kind' has no matching entity type\n";
    }
}

#----------------------------------------------------------------------

sub get_reference_name {
    my(%params) = @_;

    my $reference_name = $params{reference_name};

    if (!%$reference_name_map) {
        my $refs_map = $F5::ASMConfig::Handler::Rest::path_mapper->{references}->{policies}->{references};
         for my $reference (sort keys %$refs_map) {
            if ($refs_map->{$reference}->{reference_name}) {
                $reference_name_map->{$refs_map->{$reference}->{reference_name}} = $reference;
            }
        }
    }

    if ( $reference_name_map->{$reference_name} ) {
        return $reference_name_map->{$reference_name};
    }
    else {
        return;
    }
}

#----------------------------------------------------------------------

sub item_references_fixup {
    my(%params) = @_;

    my $item = $params{item};
    my $entity_type = $params{entity_type};
    my $direction = $params{direction};

    if ($direction eq 'to_declarative') {
        delete $item->{link};
    }

    my @policy_references;

    my $entity_external_structs = get_entity_structs(entity_type => $entity_type);
    for my $external_struct (values %$entity_external_structs) {
        my $rest_field_name = $external_struct->{rest_field_name};
        if ($direction eq 'to_declarative' && $external_struct->{rest_expand_name}) {
            $rest_field_name = $external_struct->{rest_expand_name};
        }

        my $values;
        if (my $rest_group = $external_struct->{rest_group}) {
            $values = $external_struct->{nested_struct} ? $item->{$rest_group}->{$rest_field_name} : [ $item->{$rest_group} ];
        }
        else {
            $values = $external_struct->{nested_struct} ? $item->{$rest_field_name} : [ $item ];
        }

        for my $value (@$values) {
            if ($direction eq 'from_declarative') {
                my $rv = from_declarative(value => $value,
                                          field_def => $external_struct,
                                          entity_type => $entity_type);
                if ($rv && $rv->{policy_reference}) {
                    push (@policy_references, $rv->{policy_reference});
                }
            }
            else {
                to_declarative(value => $value,
                               field_def => $external_struct);
            }
        }
    }

    return { policy_references => \@policy_references };
}

#----------------------------------------------------------------------

sub to_declarative {
    my(%params) = @_;

    my $value = $params{value};
    my $field_def = $params{field_def};

    my $unique_field = $field_def->{rest_unique_field_name};

    if (my $rest_expand_name = $field_def->{rest_expand_name}) {

        return if !$value->{$rest_expand_name};

        my $rest_field_name = $field_def->{rest_field_name};
        if ($unique_field) {
            $value->{$unique_field} = $value->{$rest_expand_name}->{$unique_field};
            delete $value->{$rest_expand_name};
        }
        else {
            $value->{$rest_field_name} = $value->{$rest_expand_name};
            delete $value->{$rest_expand_name};
            delete $value->{$rest_field_name}->{link};
        }
    }
    elsif (my $rest_sub_field_expand_name = $field_def->{rest_sub_field_expand_name}) {

        return if !$value->{$rest_sub_field_expand_name};

        my $rest_sub_field_name = $field_def->{rest_sub_field_name};
        if ($unique_field) {
            $value->{$unique_field} = $value->{$rest_sub_field_expand_name}->{$unique_field};
            if (exists $value->{$rest_sub_field_expand_name}->{signatureId}) {
               $value->{name} = $value->{$rest_sub_field_expand_name}->{name};
               if ($value->{$rest_sub_field_expand_name}->{isUserDefined}) {
                  $value->{tag} = $value->{$rest_sub_field_expand_name}->{tag};
                  delete $value->{$unique_field};
               }
            }
            delete $value->{$rest_sub_field_expand_name};
        }
        else {
            $value->{$rest_sub_field_name} = $value->{$rest_sub_field_expand_name};
            delete $value->{$rest_sub_field_expand_name};
            delete $value->{$rest_sub_field_name}->{link};
        }
    }
}

#----------------------------------------------------------------------

sub from_declarative {
    my(%params) = @_;

    my $value = $params{value};
    my $field_def = $params{field_def};
    my $entity_type = $params{entity_type};

    return if $field_def->{skip_declarative_fixup};

    if($field_def->{rest_field_name} eq 'signatureOverrides' ||
       $field_def->{rest_field_name} eq 'signature'){
        if ( $value && !defined $value->{signatureId} && defined $value->{name} ) {

             $value->{signature}->{name} = $value->{name};
             $value->{signature}->{tag} = $value->{tag} || '';

             delete $value->{name};
             delete $value->{tag};
        }
    }

    if ($field_def->{declarative_fixup}) {
        return $entity_type->from_declarative(object => $value,
                                              field_name => $field_def->{field},
                                              field_def => $field_def,);
    }

    my $unique_field = $field_def->{rest_unique_field_name};
    my $rest_field_name = $field_def->{rest_field_name};

    if (my $rest_sub_field_name = $field_def->{rest_sub_field_name}) {
        if ($unique_field) {
            return if !defined $value->{$unique_field};
            $value->{$rest_sub_field_name}->{$unique_field} = $value->{$unique_field};
            delete $value->{$unique_field};
        }
        else {
            return if !defined $value->{$rest_sub_field_name};
        }
    }
    elsif ($unique_field) {
        return if !defined $value->{$unique_field};
        $value->{$rest_field_name}->{$unique_field} = $value->{$unique_field};
        delete $value->{$unique_field};
    }
    elsif (!$value->{$rest_field_name}) {
        return;
    }

    return { policy_reference => $field_def->{policy_reference} };
}

#----------------------------------------------------------------------

sub yaml_to_json {
    my(%params) = @_;

    my $encoding = $params{encoding};
    my $filename = $params{filename};

    open(my $contFH, '-|', "$YAML_TO_JSON_APP --encoding $encoding $filename 2>&1");

    my $json_cont = '';
    my $line;
    while (defined($line = <$contFH>)) {
        $json_cont .= $line;
    }
    close $contFH;

    my $rv = {};
    my $exitcode = $? >> 8;
    if ($exitcode) {
        $rv->{completed_successfully} = 0;
        $rv->{error_message} = $json_cont;
    }
    else {
        $rv->{completed_successfully} = 1;
        $rv->{data} = $json_cont;
    }

    return $rv;
}

#----------------------------------------------------------------------


1;
